function normals = computeNormals(X, neighbors, robust)
%COMPUTENORMALS Compute normals of vertices X with neighbors NEIGHBORS.
%
%   COMPUTENORMALS(X, NEIGHBORS) computes an Nx3 matrix NORMALS for each
%   vertex in the Nx3 coordinate matrix X. A normal vector is computed as
%   the corresponding eigenvector to the smallest eigenvalue when
%   decomposing the moment matrix of neighboring vertices.
%
%   COMPUTENORMALS(..., ROBUST) takes the median normal direction of
%   neighboring vertices if ROBUST > 0. This robust averaging is repeated
%   ROBUST times.
%
%   Contact person:
%     Falko Schindler (falko.schindler@uni-bonn.de)
%
%   Copyright 2011-2012

%% compute normals via eigenvalue decompositions
normals = zeros(size(X));
I = size(X, 1);
for i = 1 : I
    x = X(neighbors{i}, :);
    x0 = mean(x);
    x_ = x - x0(ones(size(x, 1), 1), :);
    [V, D] = eig(x_' * x_); %#ok<NASGU>
    normals(i, :) = V(:, 1) * sign(dot(V(:, 1), X(i, :)));
end

%% robustify via repeated medians
if ~exist('robust', 'var'), return, end
for r = 1 : robust
    normals = cell2mat(arrayfun(@(i) ...
        median(normals([i, neighbors{i}], :)), ...
        1 : I, 'UniformOutput', 0)');
    normals = normals ./ sqrt(normals.^2 * ones(3));
end
