function label = fastfps(neighbors, features, varargin)
%FASTFPS Perform segmentation using fast marching farthest point sampling.
%
%   FASTFPS(NEIGHBORS, FEATURES, L) computes a segmentation of a
%   triangulation given as cell array of NEIGHBORS, with one cell for each
%   vertex containing all indices of neighboring vertices. The segmentation
%   boundaries propagate according to similarity of vertex FEATURES, given
%   as Nx3 matrix with one row for each vertex. The segmentation evaluates
%   Euclidean distances in feature space. It incrementally adds new
%   segments, until L segments have been created.
%
%   FASTFPS(NEIGHBORS, FEATURES, L0, L) incrementally adds L0 segments and
%   decrementally removes segments until L final segments are obtained.
%
%   FASTFPS(NEIGHBORS, FEATURES, X, SIGMA) automatically determines the
%   optimal number of segments evaulating the planarity of each segment.
%   Therefore the Nx3 vertex coordinates X and the coordinate accuracy
%   SIGMA are needed. Note that this approach is designed for piece-wise
%   planar surfaces only.
%
%   FASTFPS(..., FLAT) samples seed points for new segments only at FLAT
%   vertices. FLAT can be a logical vector of length N or a list of vertix
%   indices. This prevents from sampling on sharp edges which often leads
%   to very small segments that have to be removed during decremental
%   segmentation.
%
%   For detailed information please refer to:
%
%     @inproceedings{Schindler*11:fast,
%       author = {Schindler, Falko and F\"orstner, Wolfgang},
%       booktitle = {LNCS, Photogrammetric Image Analysis},
%       title = {{Fast Marching for Robust Surface Segmentation}},
%       year = {2011}
%     }
%
%   Authors:
%     Falko Schindler, Wolfgang Foerstner
%     Department of Photogrammetry
%     Institute of Geodesy and Geoinformation
%     University of Bonn
%     Bonn, Germany
%
%   Contact person:
%     Falko Schindler (falko.schindler@uni-bonn.de)
%
%   Copyright 2011-2012

%% parse arguments
if isscalar(varargin{1})
    L0 = varargin{1};
    if numel(varargin) > 1 && isscalar(varargin{2})
        L = varargin{2};
    else
        L = L0;
    end
else
    X = varargin{1};
    sigma = varargin{2};
end
if numel(varargin) && isvector(varargin{end}) && ~isscalar(varargin{end})
    flat = varargin{end};
else
	flat = 1 : numel(neighbors);
end
if islogical(flat)
    flat = find(flat);
end

%% initialization
rand('seed', 1);
N = numel(neighbors);
dist = inf(N, 1);
label = nan(N, 1);
T_inc = 10;
T_dec = 1.1;

%% incremental fastfps
l = 0;
while ~exist('L0', 'var') || l < L0
    
    l = l + 1;
    
    % new front, i. e. new seed (only in flat areas)
    front = flat(dist(flat) == max(dist(flat)));
    if numel(front) > 1, front = randsample(front, 1); end
    dist(front) = 0;
    label(front) = l;

    % run fast marching
    [dist, label] = fastfpsMarch(dist, label, front, features, neighbors);
    
    % check objective function
    if ~exist('L0', 'var')
        [Phi(l), Omega(l)] = objective(X, label, sigma); %#ok<AGROW>
        if l > 1 && ...
            Omega(l) < T_inc * N * sigma^2 && ...
            diff(Phi(end - 1 : end)) > 0 && ...
            diff(Omega(end - 1 : end)) < 0
            L0 = 3 * l;
        end
    end

end
fprintf('Incremental segmentation stopped at %d segments.\n', l);

%% decremental fastfps
l = L0;
while (~exist('L', 'var') || l > L) && l > 1

    % sort by triangles per label
    [val, idx] = sort(arrayfun(@(l) sum(label == l), 1 : l), 'descend');
    [tmp, idx] = unique(idx);
    label(~isnan(label)) = idx(label(~isnan(label)))';
    
    % new front, i. e. neighbors of obsolete triangles
    obsolete = find(label == l);
    front = setdiff(reshape([neighbors{label == l}], [], 1), [0; obsolete])';
    dist(obsolete) = inf;
    label(obsolete) = nan;
    
    % run fast marching
    [dist, label] = fastfpsMarch(dist, label, front, features, neighbors);
    
    % check objective function
    l = l - 1;
    if ~exist('L', 'var')
        Phi(l) = objective(X, label, sigma); %#ok<AGROW>
        if l < L0 - 1 && Phi(l) > Phi(l + 1) * T_dec
            label = label_old;
            [L, l] = deal(l + 1);
        end
    end
    label_old = label;

end
fprintf('Decremental segmentation stopped at %d segments.\n', l);
