function [dist, label] = fastfpsMarch( ...
    dist, label, front, features, neighbors)
%FASTFPSMARCH Perform on marching step of the FastFPS algorithm.
%
%   Given the current distance map DIST, the current labeling LABEL, an
%   index list of the current FRONT, point FEATURES and point-to-point
%   NEIGHBORS, march until the front is empty.
%
%   For detailed information please refer to:
%
%     @inproceedings{Schindler*11:fast,
%       author = {Schindler, Falko and F\"orstner, Wolfgang},
%       booktitle = {LNCS, Photogrammetric Image Analysis},
%       title = {{Fast Marching for Robust Surface Segmentation}},
%       year = {2011}
%     }
%
%   Contact person:
%     Falko Schindler (falko.schindler@uni-bonn.de)
%
%   Copyright 2011-2012

%% initialize front
F = false(numel(dist), 1);
F(front) = true;

%% march
while ~isempty(front)

    % remove vertex u with minimal distance from front F
    u = front(dist(front) == min(dist(front)));
    u = u(1);
    if isinf(dist(u)), break; end
    F(u) = false;
    front(front == u) = [];

    % update neighbors
    v = neighbors{u};
    v = v(~~v);
    newDist = dist(u) + ...
        sqrt(nansum((features(u(ones(1, numel(v))), :) - features(v, :)).^2, 2));
    sel = newDist < dist(v);
    dist(v(sel)) = newDist(sel);
    label(v(sel)) = label(u);
    F(v(sel)) = true;
    front = [front, v(sel)]; %#ok<AGROW>

end
