function [Phi, Omega] = objective(X, label, sigma)
%OBJECTIVE Evaluate the objective function for current labeling.
%
%   Given the Nx3 vertex coordinate matrix X, the current labeling LABEL
%   and the vertex coordinate accuracy, evaluate the potentials PHI and sum
%   of squared residuals OMEGA for the assumption of a piece-wise planar
%   surface.
%
%   For detailed information please refer to:
%
%     @inproceedings{Schindler*11:fast,
%       author = {Schindler, Falko and F\"orstner, Wolfgang},
%       booktitle = {LNCS, Photogrammetric Image Analysis},
%       title = {{Fast Marching for Robust Surface Segmentation}},
%       year = {2011}
%     }
%
%   Contact person:
%     Falko Schindler (falko.schindler@uni-bonn.de)
%
%   Copyright 2011-2012

%% dimensions
S = max(label);
N = size(X, 1);
R = max(range(X));
eps = sigma / 10;

%% labeling
Phi_label = N * log2(S);

%% planes
Phi_planes = 3 * S * log2(R / eps);

%% points
Phi_points = 0;
Omega = 0;
for s = 1 : S
    n = label == s;
    Xm = mean(X(n, :), 1);
    X0 = [X(n, :) - Xm(ones(sum(n), 1), :), ones(sum(n), 1)];
    [U, D] = eig(X0' * X0); %#ok<NASGU>
    X_ = X0 * U;
    v = X_(:, 1) ./ X_(:, 4);
    Phi_points = Phi_points + 2 * N * log2(R / eps) + ...
        sum(v.^2 / 2 / log(2) / sigma^2 + log2(sqrt(2 * pi) * sigma / eps));
    Omega = Omega + sum(v.^2);
end

%% sum
Phi = Phi_label + Phi_planes + Phi_points;
